

import subprocess
import tempfile
from enum import Enum
import re
import sys
import questionary
import click
import subprocess_tee
from . import chatter
from . import config


def extract_arguments(code):
    matches = re.findall(r"\$[0-9]+", code)
    return matches


def require_args_if_need(code):
    args = extract_arguments(code)
    if len(args) > 0:
        args_string = questionary.text(f"Please input arguments:").ask()
        return split_by_whitespace(args_string)
    return []


def retry_if_fail(func):
    def wrapper(*args, **kwargs):
        while True:
            try:
                return func(*args, **kwargs)
            except:
                if questionary.confirm("Oops, something went wrong. Retry?").ask():
                    pass
                else:
                    sys.exit(0)
    return wrapper


def execute_bash_tee(filename, args=[]):
    return subprocess_tee.run(["bash", filename] + args)


def execute_bash(filename, args=[]):
    return subprocess.run(["bash", filename] + args)


def create_tmp_file(code):
    with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
        tmp_file.write(code.encode("utf-8"))
        return tmp_file.name


class NextAction(Enum):
    DO = "Do"
    SAVE = "Save"
    ADDITONAL_REQUEST = "Additonal Request"
    ANOTHER_QUESTION = "Another Question"
    EXIT = "Exit"


def split_by_whitespace(string):
    regex = re.compile(r'\s+')
    return regex.split(string)


def call_whats_next():
    na = questionary.select(
        "What's next?",
        choices=[e.value for e in NextAction],
    ).ask()
    return NextAction(na)


def try_save(agent):
    retry_think_script_name = retry_if_fail(lambda: agent.think_script_names())

    names = retry_think_script_name()
    selected_name = questionary.select(
        "I thought of the following names. Which one do you like?",
        choices=names,
    ).ask()
    summary = agent.question_summary()
    click.echo(f"Saved as {selected_name}: Description:{summary}")
    config.save_script(selected_name, agent.code, summary)


def conversation_cycle(agent, is_display_comment=True):
    display_comment = agent.display_comment
    code = agent.code

    if is_display_comment:
        if code is None:
            click.echo(click.style("AI Answer:",
                                   bold=True, underline=True, reverse=True))
            click.echo(display_comment)
            click.echo(click.style("Sorry, I could not generate the proper code from your comment.",
                                   fg="red", bold=True))
            sys.exit(0)

        click.echo(click.style("AI Answer:", bold=True, reverse=True))
        click.echo(display_comment)

    next_action = call_whats_next()

    if next_action == NextAction.DO:
        tmp = create_tmp_file(code)
        args = require_args_if_need(code)
        result = execute_bash_tee(tmp, args)
        agent.report_result(result)
        if result.returncode != 0 or len(result.stderr) > 0:
            think_again = questionary.confirm(
                "Something doesn't seem to be going right. \nCan I try to fix the code?", default=False).ask()
            if think_again:
                agent.think_script("")
                return conversation_cycle(agent, is_display_comment=True)

        return conversation_cycle(agent, is_display_comment=False)

    elif next_action == NextAction.SAVE:
        return try_save(agent)

    elif next_action == NextAction.ADDITONAL_REQUEST:
        text = questionary.text("What additional requests do you have?").ask()
        agent.think_script(text)
        return conversation_cycle(agent)

    elif next_action == NextAction.ANOTHER_QUESTION:
        return what_wanna_do(agent.question_summary())

    elif next_action == NextAction.EXIT:
        sys.exit(0)


def what_wanna_do(default_question=""):
    question = questionary.text(
        'What do you want to do ?', default=default_question).ask()

    if len(question.strip()) == 0:
        click.echo("please tell me what you wanna do", err=True)
        sys.exit(0)
    agent = chatter.BashAgent()
    agent.think_script(question)

    conversation_cycle(agent)


@click.group()
def cmd():
    """Shell command launcher with natural language"""
    pass


def fill_command(command):
    if config.exists(command):
        return command
    ideas = config.ideas()
    command_and_question = questionary.autocomplete(
        'Choose command',
        choices=ideas).ask()
    got_command = command_and_question.split(":")[0].strip()
    if config.exists(got_command):
        return got_command
    else:
        sys.exit(0)


def fill_args(cmd):
    return require_args_if_need(config.get_code(cmd))


@cmd.command()
@click.argument('command', required=False)
@click.argument("args", nargs=-1)
def do(command=None, args=None):
    """Execute a command"""
    cmd = fill_command(command)
    args = [a for a in args] if args is not None else []
    filled_args = args if command else fill_args(cmd)

    execute_bash(config.get_command_path(cmd), filled_args)


@cmd.command()
@click.argument('question', required=False)
def think(question=None):
    """Generate a bash script that answers the incoming request"""
    if (question is None):
        what_wanna_do()
    else:
        what_wanna_do(question)


@cmd.command()
@click.option('--command', is_flag=True)
def list(command):
    """List up all commands"""

    target_list = config.commands() if command else config.ideas()

    for value in target_list:
        click.echo(value)


@cmd.command()
@click.argument('command', required=False)
@click.argument("args", nargs=-1)
def remove(command, args):
    """Remove selected command"""
    click.echo("Removed:" + command)
    cmd = fill_command(command)
    config.remove(cmd)


def main():
    cmd()


if __name__ == '__main__':
    main()



import os
import openai
import json
import re
import sys
from subprocess_tee import run
from glom import glom
from . import system
from . import codedisplay


if "OPENAI_API_KEY" not in os.environ:
    print("ERROR: OPENAI_API_KEY environment variable not found.")
    sys.exit(1)

openai.api_key = os.environ["OPENAI_API_KEY"]


BASH_SCRIPT_PROMPT = """
If someone asks you to perform a task, your job is to come up with a series of bash scripts that will perform that task.
Do not use anything but bash scripts.Take up to two arguments if necessary.
Use the following format and try to explain your reasoning step by step:.

Q: "Copy files in a directory named "target" to a new directory at the same level as target named "myNewDirectory"."

The following actions are required
- Create a nepow directory
- Copy files from the first directory to the second
```bash
#!/usr/bin/env bash
mkdir myNewDirectory
cp -r target/* myNewDirectory
````

"""

LIST_UP_FILENAMES_PROMPT = """
If you were to name this bashscript, what file name would you give it?
Be sure to output 4 candidates
Please follow the example name without the ".sh" and answer with a list of four choices in json format. (ex.["good_one", "something_special", "count_records", "delete_files"])
"""

SUMMARY_PROMPT = """
Please write a description of this bash script.
Condition 1: Must be less than 100 characters
Condition 2: Output should be in the same language as the user's input language.
Condition 3: The description of the operation should be simple.
Condition 4: It is obvious that it is a bash script, so it is not mentioned.
The instructions so far are as follows:
"""


def extract_json_block(text):
    code_regex = r"```json([\s\S]*?)```"
    code_matches = re.findall(code_regex, text)

    if code_matches:
        return code_matches[0]
    else:
        return text


def parse_json(text):
    return json.loads(extract_json_block(text))


def extract_head_tail(text):
    if len(text) <= 200:
        return text
    else:
        head = text[:100]
        tail = text[-100:]
        return head + "..." + tail


class BashAgent():
    PRESET_MESSAGES = [
        {"role": "system", "content": BASH_SCRIPT_PROMPT},
        {"role": "system", "content": "The operating environment for bash is as follows:"+system.info()},
    ]

    def __init__(self, temperature=0.1):
        self.temperature = temperature
        self.messages = self.PRESET_MESSAGES
        self.display_comment = ""
        self.question = []
        self.names = []
        self.code = ""

    def add_user_message(self, message):
        self.messages.append({"role": "user", "content": message})

    def add_system_message(self, message):
        self.messages.append({"role": "system", "content": message})

    def add_assistant_message(self, message):
        self.messages.append({"role": "assistant", "content": message})

    def report_result(self, result):
        self.add_system_message(f"""
        returncode : {result.returncode}
        stdout :
        {extract_head_tail(result.stdout)}
        stderr : 
        {extract_head_tail(result.stderr)}
        """)

    def chat(self):
        response = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            messages=self.messages,
            temperature=self.temperature,
        )
        message = glom(response, "choices.0.message.content", default=None)
        self.add_assistant_message(message)
        return message

    def think_script(self, question):
        self.question.append(question)
        self.add_user_message(question)
        message = self.chat()
        self.display_comment = codedisplay.highlight_code_block(message)
        self.code = codedisplay.extract_code_block(message)
        return message

    def think_script_names(self):
        self.add_system_message(LIST_UP_FILENAMES_PROMPT)
        message = self.chat()
        self.names = parse_json(message)
        return self.names

    def question_summary(self):
        if len(self.question) == 0:
            return ""
        if len(self.question) == 1:
            return self.question[0]

        self.add_system_message(SUMMARY_PROMPT)
        self.add_system_message("\n".join(self.question))
        message = self.chat()
        return message

    def reset(self):
        self.messages = self.PRESET_MESSAGES




import platform


def info():
    return f"""
        OS Name     : {platform.system()}
        OS Version  : {platform.release()}
        OS Arch     : {platform.machine()}
    """


if __name__ == "__main__":
    print(info())



import re
from pygments import highlight
from pygments.lexers import BashLexer
from pygments.formatters import TerminalFormatter


# BashLexerを使用してトークン化
lexer = BashLexer()
# TerminalFormatterを使用してターミナル用にフォーマット
formatter = TerminalFormatter(bg='dark')


def highlight_bash(code):
    lexer = BashLexer()
    formatter = TerminalFormatter(bg='dark')
    highlighted_code = highlight(code, lexer, formatter)
    return highlighted_code


def highlight_code_block(text):
    target = text
    code_regex = r"```bash([\s\S]*?)```"

    def replace_code(match):
        code = match.group(1)
        return "\n```bash\n" + highlight_bash(code) + "```"

    return re.sub(code_regex, replace_code, target)


def extract_code_block(text):
    code_regex = r"```bash([\s\S]*?)```"
    code_matches = re.findall(code_regex, text)

    if code_matches:
        return code_matches[0]
    else:
        return None
